extends Control

var escape_mode = false
var global

var boost_acc = 0.0
var is_boosting = false

var fall_acc = 0.0
var is_falling = false

var catch_up_acc = 0.0
var is_catching_up = false

var math_fn = preload("res://scripts/native/MathFn.gdns").new()
var delta_since_last_land = 1.0
var delta_since_last_boost = 1.0

const BOOST_DURATION = 1.5
const FALL_DURATION = 0.5
const CATCH_UP_DURATION = 0.5

const CAMERA_V_OFFSET_FACTOR = 0.3

const LAND_SHAKE_AMPLITUDE = 0.05
const LAND_SHAKE_DURATION = 0.3
const LAND_SHAKE_ITERATIONS = 5

const BOOST_SHAKE_AMPLITUDE = 0.2
const BOOST_SHAKE_DURATION = 0.3
const BOOST_SHAKE_ITERATIONS = 5

export var jump_button_relative_size = 0.25
export var jump_margin_relative_size = 0.05

# Called when the node enters the scene tree for the first time.
func _ready():
	global = get_node("/root/Global")
	$Tunnel.setup("res://scenes/Rail.tscn", global.skill)
	$Input.set_jump_stickiness_factor(1.0)
	$Input.set_mouse_sensibility_factor(1.0)
	$Input.set_keyboard_sensibility_factor(1.0)
	$Effects/CRT.visible = false
	if $EscBar/Buttons/EscButton.connect("Esc", self, "on_escape"):
		pass
	if get_tree().get_root().connect("size_changed", self, "setup_layout"):
		pass
	setup_layout()

func setup_layout():
	var viewport_rect = get_viewport_rect().size
	var base_size = (viewport_rect.x + viewport_rect.y)/2
	var button_size = Vector2(
		base_size * jump_button_relative_size,
		base_size * jump_button_relative_size)
	var margin_size = Vector2(
		viewport_rect.x * jump_margin_relative_size,
		viewport_rect.y * jump_margin_relative_size)
	var size1 = $MobileUI/JumpButton/JumpButton1.rect_size
	var scale1 = Vector2(button_size.x/size1.x,button_size.y/size1.y)
	var size2 = $MobileUI/JumpButton/JumpButton2.rect_size
	var scale2 = Vector2(button_size.x/size2.x,button_size.y/size2.y)
	var position = Vector2(margin_size.x, viewport_rect.y-(button_size.y+margin_size.y))
	$MobileUI/JumpButton/JumpButton1.rect_position = position
	$MobileUI/JumpButton/JumpButton2.rect_position = position
	$MobileUI/JumpButton/JumpButton1.rect_scale = scale1
	$MobileUI/JumpButton/JumpButton2.rect_scale = scale2
	$Input.set_jump_button(position, size1, scale1)
	$Input.set_viewport_rect(viewport_rect)
	update_score()

func update_score():
	var viewport_rect = get_viewport_rect().size
	$UI/Score.text=str($Ball.score())
	$UI/Score.rect_position.y = 0
	$UI/Score.rect_position.x = viewport_rect.x-$UI/Score.rect_size.x

func on_escape():
	$Input.set_escape_mode(true)

func _process(delta):
	escape_mode = $Input.get_escape_mode()
	$UI/Escape.visible = escape_mode
	$EscBar/Buttons.visible = (not escape_mode) and global.show_mobile_ui
	var player_position=$Ball.position()
	var best_row=$Ball.best_row()
	$Tunnel.update_player_position(player_position, best_row)
	$Tunnel.set_level_row($Ball.level_row())
	$Tunnel.set_level_col($Ball.level_col($Tunnel.width()))
	$Ball.update_player_position(player_position)
	update_score()
	update_boost(delta)
	update_fall(delta)
	update_catch_up(delta)
	update_camera(delta)

func update_camera(delta):
	var last_land_v_offset=LAND_SHAKE_AMPLITUDE*math_fn.fade_out(delta_since_last_land/LAND_SHAKE_DURATION)*math_fn.triangle_wave(delta_since_last_land*LAND_SHAKE_ITERATIONS)
	var ball_position_v_offset=CAMERA_V_OFFSET_FACTOR*$Ball.position().y
	$MainCamera.v_offset = last_land_v_offset + ball_position_v_offset
	var last_boost_h_offset=BOOST_SHAKE_AMPLITUDE*math_fn.fade_out(delta_since_last_boost/BOOST_SHAKE_DURATION)*math_fn.triangle_wave(delta_since_last_boost*BOOST_SHAKE_ITERATIONS)
	$MainCamera.h_offset = last_boost_h_offset
	delta_since_last_land += delta
	delta_since_last_boost += delta

func update_boost(delta):
	if not is_boosting:
		$PreEffects/BoostFlash.visible=false
		return
	if boost_acc>1.0:
		is_boosting = false
		$PreEffects/BoostFlash.visible=true
		return
	boost_acc += delta/BOOST_DURATION
	$PreEffects/BoostFlash.visible=true
	$PreEffects/BoostFlash.material.set_shader_param("blend", boost_acc)

func update_fall(delta):
	if not is_falling:
		$PreEffects/FallFlash.visible=false
		return
	if fall_acc>1.0:
		is_falling = false
		$PreEffects/FallFlash.visible=true
		return
	fall_acc += delta/FALL_DURATION
	$PreEffects/FallFlash.visible=true
	$PreEffects/FallFlash.material.set_shader_param("blend", fall_acc)

func update_catch_up(delta):
	if not is_catching_up:
		$PreEffects/CatchUpFlash.visible=false
		return
	if catch_up_acc>1.0:
		is_catching_up = false
		$PreEffects/CatchUpFlash.visible=true
		return
	catch_up_acc += delta/CATCH_UP_DURATION
	$PreEffects/CatchUpFlash.visible=true
	$PreEffects/CatchUpFlash.material.set_shader_param("blend", catch_up_acc)

func _physics_process(delta):
	if escape_mode:
		return
	$Ball.tick(delta, $Tunnel, $Input)

func _on_Play_resume():
	$Input.set_escape_mode(false)

func _on_Menu_back():
	$Input.set_escape_mode(false)
	if get_tree().change_scene("res://scenes/MainMenu.tscn"):
		pass

func _notification(what):
	if what == MainLoop.NOTIFICATION_WM_GO_BACK_REQUEST || what == MainLoop.NOTIFICATION_WM_QUIT_REQUEST:
		$Input.set_escape_mode(true)

func _on_Ball_jump():
	$SFX/Jump.play()

func _on_Ball_land():
	delta_since_last_land = 0.0
	$SFX/Land.play()

func _on_Ball_fall():
	$Effects/CRT.visible = true
	$SFX/Fall.play()
	fall_acc = 0.0;
	is_falling = true;
	
func _on_Ball_start():
	$SFX/Start.play()

func _on_Ball_catch_up():
	$Effects/CRT.visible = false
	$SFX/CatchUp.play()
	catch_up_acc = 0.0;
	is_catching_up = true;

func _on_Ball_change_column():
	$SFX/ChangeColumn.play()

func _on_Ball_boost():
	delta_since_last_boost = 0.0
	$SFX/Boost.play()
	boost_acc = 0.0;
	is_boosting = true;
